import React, {useCallback, useMemo, useState} from "react";
import {Slate, withReact, Editable} from "slate-react";
import {
    createEditor, Descendant,
    Element as SlateElement
} from "slate";
import {Toolbar} from "./EditorComponents.tsx";
import {css} from "@emotion/css";

const initialValue: any[] = [
    {
        type:'heading-one',
        children:[
            {text:'111'}
        ]
    },
    {
        type: 'paragraph',
        children: [
            {text: 'This is editable '},
            {text: 'rich', bold: true},
            {text: ' text, '},
            {text: 'much', italic: true},
            {text: '测试一下', replace: true},
            {text: ' better than a '},
            {text: '<textarea>', code: true},
            {text: '!'},
            {text: '测试一下', replace: true},
        ],
    },
    {
        type: 'paragraph',
        children: [
            {
                text:
                    "Since it's rich text, you can do things like turn a selection of text ",
            },
            {text: 'bold', bold: true},
            {
                text:
                    ', or add a semantically rendered block quote in the middle of the page, like this:',
            },
        ],
    },
    {
        type: 'block-quote',
        children: [{text: 'A wise quote.'}],
    },
    {
        type: 'paragraph',
        align: 'center',
        children: [
            {text: '测试一下', replace: true},
            {text: 'Try it out for yourself!'}
        ],
    },
]
const Element = ({attributes, children, element}: any) => {
    const style = {textAlign: element.align}
    switch (element.type) {
        case 'block-quote':
            return (
                <blockquote style={style} {...attributes}>
                    {children}
                </blockquote>
            )
        case 'bulleted-list':
            return (
                <ul style={style} {...attributes}>
                    {children}
                </ul>
            )
        case 'heading-one':
            return (
                <h1 style={style} {...attributes}>
                    {children}
                </h1>
            )
        case 'heading-two':
            return (
                <h2 style={style} {...attributes}>
                    {children}
                </h2>
            )
        case 'list-item':
            return (
                <li style={style} {...attributes}>
                    {children}
                </li>
            )
        case 'numbered-list':
            return (
                <ol style={style} {...attributes}>
                    {children}
                </ol>
            )
        default:
            return (
                <p style={style} {...attributes}>
                    {children}
                </p>
            )
    }
}
const Leaf = ({attributes, children, leaf}: any) => {
    if (leaf.bold) {
        children = <strong>{children}</strong>
    }

    if (leaf.code) {
        children = <code>{children}</code>
    }
    if(leaf.replace){
        children = <span className={"text-replace-item"}>{children}</span>
    }

    if (leaf.italic) {
        children = <em>{children}</em>
    }

    if (leaf.underline) {
        children = <u>{children}</u>
    }

    return <span {...attributes}>{children}</span>
}
export const SimpleSlateEditor: React.FC = () => {

    const editor = useMemo(() => withReact(createEditor()), [])
    const [value, setValue] = useState(initialValue)
    const renderElement = useCallback(props => <Element {...props} />, [])
    const renderLeaf = useCallback(props => <Leaf {...props} />, [])
    return (
        <div>
            <Slate
                editor={editor}
                value={value}
                onChange={setValue}
            >
                <Toolbar>
                    <button>xxx1</button>
                    <button>xxx2</button>
                    <button>xxx3</button>
                </Toolbar>
                <Editable
                    renderElement={renderElement}
                    renderLeaf={renderLeaf}
                    placeholder="请输入要编辑的内容"
                    autoFocus
                />
            </Slate>
            <div className={css(`
            background:#eee;
            padding:10px;
            marginTop:20px;
            border:solid 1px #999;
            border-radius:4px;
            `)}>{JSON.stringify(value)}</div>
        </div>
    );
}
