import React, {useEffect, useRef} from "react";
import './ReplaceParser.tsx'
import ReactQuill, {Quill, Range, UnprivilegedEditor} from "react-quill";
import {DeltaStatic, Sources} from 'quill'
import 'react-quill/dist/quill.snow.css';
import {EDITOR} from "./editor.ts";

type QuillEditorProps = {
    selected?: HTMLElement;
    onClick?: (e: React.MouseEvent<HTMLElement>) => void;
}
export const QuillEditor: React.FC<QuillEditorProps> = (props) => {
    const editorInstance = useRef<ReactQuill>(null);
    const demoDiv = useRef<HTMLDivElement>(null)
    const handleChange = (_value: string, _delta: DeltaStatic, _source: Sources, _editor: UnprivilegedEditor) => {
        console.log('handleChange==>', _source)
        // setContent(
        //     editor.getContents()
        // )
    }

    useEffect(() => {
        if (editorInstance.current && editorInstance.current.editor) {
            EDITOR.instance = editorInstance.current.editor;
            EDITOR.on('demo',data=>{
                console.log('demo',data)
            })
        }
    }, [editorInstance])
    useEffect(() => {
        if (demoDiv.current) {
            demoDiv.current.addEventListener('text-change', (e) => {
                console.log(e)
            })
        }
    }, [demoDiv])
    const handleSelectChange = (selection: Range, _source: Sources, _editor: UnprivilegedEditor) => {
        if (_source == 'api') {

            console.log('handleSelectChange==>', _source, selection)
        }
    }
    const handleReplaceItemClick = (node: HTMLElement) => {
        const blot = Quill.find(node)
        const offset = blot.offset(blot.scroll), length = blot.length()
        editorInstance.current?.editor?.setSelection(offset, length)
    }
    const handleEditorClick = (e: React.MouseEvent<HTMLElement>) => {
        const span = e.target as HTMLSpanElement;
        if (!span) {
            e.stopPropagation()
            e.preventDefault()
            return;
        }
        if (span.classList.contains('text-replace-item')) {
            handleReplaceItemClick(span)
        }
        if (props.onClick) {
            props.onClick(e)
        }
    }

    useEffect(() => {
        if (props.selected) {
            handleReplaceItemClick(props.selected)
        }
    }, [props])
    //  modules={{toolbar: false}}
    const modules = {
        toolbar: [
            [{'size': []}],
            ['bold', 'italic', 'underline', 'strike'],
            [{'color': []}, {'background': []}],
            [{'script': 'super'}, {'script': 'sub'}],
            [{'header': '1'}, {'header': '2'}, 'blockquote', 'code-block'],
            [{'list': 'ordered'}, {'list': 'bullet'}, {'indent': '-1'}, {'indent': '+1'}],
            [{'direction': 'rtl'}, {'align': []}],
            ['link', 'image', 'video', 'formula'],
            ['clean']
        ],
    }
    return (<div className="editor-container" onClick={handleEditorClick} ref={demoDiv}>
        <div className="editor-toolbar-container">
            <button onClick={() => {
                console.log(EDITOR)
            }}>test
            </button>
        </div>
        <ReactQuill
            ref={editorInstance}
            theme="snow"
            modules={modules}
            onChange={handleChange}
            onChangeSelection={handleSelectChange}
        />
    </div>)

}
