import {DomEditor, IDomEditor, SlateElement} from "@wangeditor/editor";
import {h, VNode} from "snabbdom";
import {WangEditorProofreadElement} from "../../../types/editor.ts";

const ELEMENT_TYPE = 'proofread';

export function withProofread<T extends IDomEditor>(editor: T) {
    const {isInline, isVoid} = editor;
    const newEditor = editor;
    newEditor.isInline = ele => {
        const type = DomEditor.getNodeType(ele) // 获取元素类型
        return type == ELEMENT_TYPE ? true : isInline(ele);// 只针对校对类型返回为内联
    }
    newEditor.isVoid = ele => {
        const type = DomEditor.getNodeType(ele)
        return type == ELEMENT_TYPE ? false : isVoid(ele);// 只针对校对类型返回为内联
    }
    return newEditor;
}

/**
 * 渲染“校对”元素到编辑器
 * @param data 附件元素，即上文的 myResume
 * @param children 元素子节点，void 元素可忽略 @param _editor 编辑器实例
 * @returns 节点(通过 snabbdom.js 的 h 函数生成)
 */
export function renderProofread(data: SlateElement, children: VNode[] | null, _editor: IDomEditor) { // , _editor: IDomEditor
    const {proofread} = data as WangEditorProofreadElement;
    return h(
        'span',
        {
            props: {
                dataSlateId: '123123'
            }, // HTML 属性，驼峰式写法
            attrs: {
                'data-proofread-id': String(proofread.id),
                'data-proofread-text': proofread.origin
            },
            attachData: {
                proofread
            },
            className: `data-proofread-item data-proofread-item-${proofread.id} data-proofread-type-${proofread.type}`,
            style: { /* 其他... */}, // style ，驼峰式写法 display: 'inline-block', margin: '0 2px',
            on: {
                process() {
                    // const path = DomEditor.getSelectedElems(_editor)
                    // DomEditor.findPath(_editor,span)
                    const path = DomEditor.findPath(_editor, data);
                    _editor.select(path)
                    _editor.insertText(proofread.text)
                    const span = this.elm as HTMLSpanElement;
                    span.classList.add('proofread-processed')
                    // if(proofread.type == 'delete'){
                    //
                    //     return;
                    // }
                    //console.log('proofread process', data, this);
                    //Transforms.select(_editor, this.elm)
                    //window.getSelection()?.selectAllChildren(span);

                    //
                    // SlateTransforms.
                },
                click(e) {
                    e.stopPropagation()
                    e.preventDefault()
                    console.log(this)

                    const span = this.elm as HTMLSpanElement;
                    const ev = new Event('proofread-click', {
                        bubbles: true
                    })
                    span.dispatchEvent(ev);
                }, /* 其他... */
            }
        },
        children
    );
}
